"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const fileutils_1 = require("@nrwl/workspace/src/utils/fileutils");
const path_1 = require("path");
const architect_1 = require("@angular-devkit/architect");
const core_1 = require("@angular-devkit/core");
const build_webpack_1 = require("@angular-devkit/build-webpack");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const node_config_1 = require("./utilities/node.config");
const config_1 = require("./utilities/config");
const normalize_1 = require("./utilities/normalize");
const node_1 = require("@angular-devkit/core/node");
const project_graph_1 = require("@nrwl/workspace/src/core/project-graph");
const buildable_libs_utils_1 = require("@nrwl/workspace/src/utils/buildable-libs-utils");
const buildable_libs_utils_2 = require("./utilities/buildable-libs-utils");
try {
    require("dotenv").config();
}
catch (e) {
}
exports.default = architect_1.createBuilder(run);
function getSourceRoot(context) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const workspaceHost = core_1.workspaces.createWorkspaceHost(new node_1.NodeJsSyncHost());
        const { workspace } = yield core_1.workspaces.readWorkspace(context.workspaceRoot, workspaceHost);
        if (workspace.projects.get(context.target.project).sourceRoot) {
            return workspace.projects.get(context.target.project).sourceRoot;
        }
        else {
            context.reportStatus("Error");
            const message = `${context.target.project} does not have a sourceRoot. Please define one.`;
            context.logger.error(message);
            throw new Error(message);
        }
    });
}
function run(options, context) {
    if (!options.buildLibsFromSource) {
        const projGraph = project_graph_1.createProjectGraph();
        const { target, dependencies } = buildable_libs_utils_2.calculateProjectDependencies(projGraph, context);
        options.tsConfig = buildable_libs_utils_1.createTmpTsConfig(options.tsConfig, context.workspaceRoot, target.data.root, dependencies);
    }
    return rxjs_1.from(getSourceRoot(context))
        .pipe(operators_1.map(sourceRoot => normalize_1.normalizeBuildOptions(options, context.workspaceRoot, sourceRoot)), operators_1.map(options => {
        let config = node_config_1.getNodeWebpackConfig(options);
        if (options.webpackConfig) {
            config = require(options.webpackConfig)(config, {
                options,
                configuration: context.target.configuration
            });
        }
        return config;
    }), operators_1.concatMap(config => build_webpack_1.runWebpack(config, context, {
        logging: stats => {
            context.logger.info(stats.toString(config.stats));
        },
        webpackFactory: require("webpack")
    })), operators_1.map((buildEvent) => {
        const api = new WorkspaceConfiguration(options, context);
        api.createPackageJson();
        api.addPackageJsonToDist();
        api.updateTsConfig();
        api.updateJestConfig();
        api.createEsLint();
        return buildEvent;
    }), operators_1.map((buildEvent) => {
        buildEvent.outfile = path_1.resolve(context.workspaceRoot, options.outputPath, config_1.OUT_FILENAME);
        return buildEvent;
    }));
}
class WorkspaceConfiguration {
    constructor(options, context) {
        this.packageJson = {
            main: "./main.js",
            private: true,
        };
        this.tsConfig = {
            "lib": ["es2017"],
            "module": "commonjs",
            "noImplicitReturns": true,
            "sourceMap": true,
            "target": "es2017",
            "esModuleInterop": true // help with jest test
        };
        this.jestConfig = `
module.exports = {
    testMatch: ['**/+(*.)+(spec|test).+(ts|js)?(x)'],
    transform: {
      '^.+\\\\.(ts|js|html)$': 'ts-jest'
    },
    resolver: '@nrwl/jest/plugins/resolver',
    moduleFileExtensions: ['ts', 'js', 'html'],
    coverageReporters: ['html']
  };
`;
        this.globalEsLint = {
            "root": true,
            "parser": "@typescript-eslint/parser",
            "parserOptions": {
                "ecmaVersion": 2018,
                "sourceType": "module",
                "project": "./tsconfig.json"
            },
            "ignorePatterns": ["**/*"],
            "plugins": ["@typescript-eslint", "@nrwl/nx"],
            "extends": [
                "eslint:recommended",
                "plugin:@typescript-eslint/eslint-recommended",
                "plugin:@typescript-eslint/recommended",
                "prettier",
                "prettier/@typescript-eslint"
            ],
            "rules": {
                "@typescript-eslint/no-explicit-any": "off",
                "@typescript-eslint/no-unused-vars": "off",
                "@typescript-eslint/explicit-member-accessibility": "off",
                "@typescript-eslint/explicit-function-return-type": "off",
                "@typescript-eslint/no-parameter-properties": "off",
                "@nrwl/nx/enforce-module-boundaries": [
                    "error",
                    {
                        "enforceBuildableLibDependency": true,
                        "allow": [],
                        "depConstraints": [
                            { "sourceTag": "*", "onlyDependOnLibsWithTags": ["*"] }
                        ]
                    }
                ]
            },
            "overrides": [
                {
                    "files": ["*.tsx"],
                    "rules": {
                        "@typescript-eslint/no-explicit-any": "off",
                        "@typescript-eslint/no-unused-vars": "off"
                    }
                }
            ]
        };
        this.options = options;
        this.context = context;
    }
    getExternalDependencies(options = this.options, context = this.context) {
        let dependencies = {};
        const buildFile = fs.readFileSync(`${options.outputPath}/main.js`, "utf8");
        let re2 = /"(.*)"/gm;
        const externalDependencies = buildFile.match(/require\("(.*)"\)/gm);
        // Get the package version from the root
        if (externalDependencies) {
            const workspacePackages = fileutils_1.readJsonFile(path_1.join(context.workspaceRoot, "package.json"));
            const dependenciesName = externalDependencies.map((x) => x.match(re2)[0].replace(/"/gm, ""));
            dependenciesName.forEach((dep) => {
                const packageIsDefined = dep in workspacePackages.dependencies;
                dependencies[dep] = packageIsDefined ? workspacePackages.dependencies[dep]
                    : "*";
            });
        }
        return {
            dependencies,
            exist: Boolean(exports)
        };
    }
    getPackageJson(options = this.options, context = this.context) {
        const exists = fileutils_1.fileExists(options.packageJson);
        return exists ? { data: fileutils_1.readJsonFile(options.packageJson), exist: true } : { exist: false };
    }
    createPackageJson() {
        const externalDependencies = this.getExternalDependencies();
        const originalPackageJson = this.getPackageJson();
        if (externalDependencies.exist) {
            this.packageJson.dependencies = externalDependencies.dependencies;
        }
        if (originalPackageJson.exist) {
            const { name, version } = originalPackageJson.data;
            this.packageJson.name = name || '';
            this.packageJson.version = version || '';
        }
    }
    addPackageJsonToDist(options = this.options, packageJson = this.packageJson) {
        fileutils_1.writeJsonFile(`${options.outputPath}/package.json`, packageJson);
    }
    updateTsConfig(options = this.options, context = this.context) {
        if (options.setRootTsConfig) {
            const path = path_1.join(context.workspaceRoot, "tsconfig.json");
            const tsConfigExist = fileutils_1.fileExists(path);
            if (tsConfigExist) {
                const original = fileutils_1.readJsonFile(path);
                for (const prop in this.tsConfig) {
                    if (this.tsConfig.hasOwnProperty(prop)) {
                        original.compilerOptions[prop] = this.tsConfig[prop];
                    }
                }
                fileutils_1.writeJsonFile(path, original);
            }
            else {
                fileutils_1.writeJsonFile(path, this.tsConfig);
            }
        }
    }
    updateJestConfig(options = this.options, context = this.context) {
        if (options.setRootJestConfig) {
            const path = path_1.join(context.workspaceRoot, "jest.config.json");
            const jestConfigExist = fileutils_1.fileExists(path);
            if (jestConfigExist) {
                fileutils_1.writeToFile(path, this.jestConfig);
            }
        }
    }
    createEsLint(options = this.options, context = this.context) {
        if (options.setRootEsLint) {
            const path = path_1.join(context.workspaceRoot, ".eslintrc");
            const requireNewFile = !fileutils_1.fileExists(path);
            if (requireNewFile) {
                fileutils_1.writeJsonFile(path, this.globalEsLint);
            }
        }
    }
}
//# sourceMappingURL=builder.js.map