"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const util_1 = require("util");
const workspace_1 = require("@nrwl/workspace");
const glob = require("glob");
const path_1 = require("path");
const buildable_libs_utils_1 = require("@nrwl/workspace/src/utils/buildable-libs-utils");
const project_graph_1 = require("@nrwl/workspace/src/core/project-graph");
const child_process_1 = require("child_process");
const fs_extra_1 = require("fs-extra");
const fileutils_1 = require("@nrwl/workspace/src/utils/fileutils");
const buildable_libs_utils_2 = require("./buildable-libs-utils");
// fixme: clean the signature & output
class ProjectInfo {
    constructor(options, context) {
        this.options = this.getOptions(options, context);
        this.context = context;
        return this;
    }
    getAll() {
        const options = this.options;
        const projectGraph = this.getProjectGraph();
        const target = this.getTarget();
        const dependencies = this.getDependencies();
        const context = this.context;
        const dependenciesAreBuilt = this.getDependenciesAreBuilt();
        return {
            options,
            dependencies,
            dependenciesAreBuilt,
            target,
            projectGraph
            // context,
        };
    }
    log(message) {
        this.context.logger.info(util_1.inspect(message, false, null));
    }
    getOptions(options, context) {
        if (this.options === undefined) {
            const outDir = options.outputPath;
            const files = [];
            const globbedFiles = (pattern, input = "", ignore = []) => {
                return glob.sync(pattern, {
                    cwd: input,
                    nodir: true,
                    ignore
                });
            };
            options.assets.forEach(asset => {
                if (typeof asset === "string") {
                    // @ts-ignore
                    globbedFiles(asset, context.workspaceRoot)
                        // @ts-ignore
                        .forEach(globbedFile => {
                        // @ts-ignore
                        files.push({
                            input: path_1.join(context.workspaceRoot, globbedFile),
                            output: path_1.join(context.workspaceRoot, outDir, path_1.basename(globbedFile))
                        });
                    });
                }
                else {
                    globbedFiles(asset.glob, path_1.join(context.workspaceRoot, asset.input), asset.ignore)
                        // @ts-ignore
                        .forEach(globbedFile => {
                        // @ts-ignore
                        files.push({
                            input: path_1.join(context.workspaceRoot, asset.input, globbedFile),
                            output: path_1.join(context.workspaceRoot, outDir, asset.output, globbedFile)
                        });
                    });
                }
            });
            // Relative path for the dist directory
            const tsconfig = workspace_1.readJsonFile(path_1.join(context.workspaceRoot, options.tsConfig));
            const rootDir = tsconfig.compilerOptions.rootDir || "";
            const mainFileDir = path_1.dirname(options.main);
            const tsconfigDir = path_1.dirname(options.tsConfig);
            const relativeMainFileOutput = path_1.relative(`${tsconfigDir}/${rootDir}`, mainFileDir);
            const watch = options.watch || false;
            const sourceMap = options.sourceMap || true;
            this.options = Object.assign(Object.assign({}, options), { files,
                watch,
                sourceMap,
                relativeMainFileOutput, normalizedOutputPath: path_1.join(context.workspaceRoot, options.outputPath) });
        }
        return this.options;
    }
    getProjectGraph() {
        if (this.projectGraph === undefined) {
            this.projectGraph = project_graph_1.createProjectGraph();
        }
        return this.projectGraph;
    }
    getTarget(projectGraph = this.getProjectGraph(), context = this.context) {
        if (this.target === undefined) {
            const { target, dependencies } = buildable_libs_utils_2.calculateProjectDependencies(projectGraph, context);
            this.target = target;
            this.dependencies = dependencies;
        }
        return this.target;
    }
    getDependencies(projectGraph = this.getProjectGraph(), context = this.context) {
        if (this.dependencies === undefined) {
            const { target, dependencies } = buildable_libs_utils_2.calculateProjectDependencies(projectGraph, context);
            this.target = target;
            this.dependencies = dependencies;
        }
        return this.dependencies;
    }
    getDependenciesAreBuilt(context = this.context, dependencies = this.getDependencies()) {
        if (this.dependenciesAreBuilt === undefined) {
            this.dependenciesAreBuilt = buildable_libs_utils_1.checkDependentProjectsHaveBeenBuilt(context, dependencies);
        }
        return this.dependenciesAreBuilt;
    }
    reportAnError(message = "", error) {
        if (message) {
            this.log(message);
        }
        if (error) {
            this.log(error);
        }
        return {
            success: false,
            info: { message },
            error
        };
    }
    copyAssetFiles(options = this.options, context = this.context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.log("Start: Copy Assets Files");
            try {
                // @ts-ignore
                const promises = options.files.map(file => fs_extra_1.copy(file.input, file.output));
                yield Promise.all(promises);
                this.log("Done: Copy Assets Files");
                return this.ok();
            }
            catch (error) {
                return this.reportAnError(error.message, error);
            }
        });
    }
    compileTypeScriptFiles(options = this.options, context = this.context, projectGraph = this.getProjectGraph(), projectDependencies = this.getDependencies()) {
        this.log("Start: Compile Typescript Files");
        const libRoot = projectGraph.nodes[context.target.project].data.root;
        let tsConfigPath = path_1.join(context.workspaceRoot, options.tsConfig);
        if (projectDependencies.length > 0) {
            tsConfigPath = buildable_libs_utils_1.createTmpTsConfig(tsConfigPath, context.workspaceRoot, libRoot, projectDependencies);
        }
        let command = ["tsc", "-p", tsConfigPath, "--outDir", options.normalizedOutputPath];
        if (options.sourceMap) {
            command.push("--sourceMap");
        }
        const tscPath = path_1.join(context.workspaceRoot, "/node_modules/typescript/bin/tsc");
        const child = child_process_1.exec(command.join(" "));
        child.stdout.on("data", data => this.log(data.toString()));
        child.stderr.on("data", data => this.log(data.toString()));
        return new Promise(resolve => {
            this.log("Done: Compile Typescript Files");
            child.on("close", code => resolve({ success: code === 0 }));
        });
    }
    updatePackageJson(options = this.options, context = this.context) {
        this.log("Start: Update Package Json");
        const mainFile = path_1.basename(options.main, ".ts");
        const typingsFile = `${mainFile}.d.ts`;
        const mainJsFile = `${mainFile}.js`;
        let packageJson = workspace_1.readJsonFile(path_1.join(context.workspaceRoot, options.packageJson));
        packageJson.main = path_1.normalize(`./${options.relativeMainFileOutput}/${mainJsFile}`);
        packageJson.typings = path_1.normalize(`./${options.relativeMainFileOutput}/${typingsFile}`);
        return new Promise(resolve => {
            fileutils_1.writeJsonFile(`${options.outputPath}/package.json`, packageJson);
            this.log("Done: Update Package Json");
            return this.ok();
        });
    }
    ok(values) {
        return Object.assign({ success: true }, values);
    }
    updateBuildableProjectPackageJsonDependencies(context = this.context, target = this.getTarget(), dependencies = this.getDependencies()) {
        buildable_libs_utils_1.updateBuildableProjectPackageJsonDependencies(context, target, dependencies);
        // fixme: redo everywhere
        return Promise.resolve(this.ok());
    }
}
exports.ProjectInfo = ProjectInfo;
//# sourceMappingURL=utility.js.map