"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const util_1 = require("util");
const workspace_1 = require("@nrwl/workspace");
class ProjectTools {
    constructor(userOptions, context) {
        this.context = context;
        this.userOptions = userOptions;
        this.options = this.normalizeOptions(userOptions);
        this.project = {
            root: this.options.appProjectRoot,
            sourceRoot: core_1.join(this.options.appProjectRoot, "src"),
            projectType: "application",
            prefix: this.options.name,
            schematics: {},
            architect: {}
        };
        return this;
    }
    normalizeOptions(userOptions) {
        const linter = userOptions.linter || "eslint" /* EsLint */;
        const projectType = workspace_1.ProjectType.Application;
        const name = workspace_1.toFileName(userOptions.name);
        const projectDirectory = userOptions.directory
            ? `${workspace_1.toFileName(userOptions.directory)}/${name}`
            : name;
        const projectName = projectDirectory.replace(new RegExp("/", "g"), "-");
        const projectRoot = `${workspace_1.projectRootDir(projectType)}/${projectDirectory}`;
        const parsedTags = userOptions.tags
            ? userOptions.tags.split(",").map(s => s.trim())
            : [];
        const className = workspace_1.toClassName(userOptions.name);
        const propertyName = workspace_1.toPropertyName(userOptions.name);
        const appDirectory = userOptions.directory
            ? `${workspace_1.toFileName(userOptions.directory)}/${workspace_1.toFileName(userOptions.name)}`
            : workspace_1.toFileName(userOptions.name);
        const appProjectRoot = core_1.join(core_1.normalize("apps"), appDirectory);
        const dot = ".";
        return Object.assign(Object.assign({}, userOptions), { linter,
            projectName,
            appDirectory,
            appProjectRoot,
            projectRoot,
            projectDirectory,
            parsedTags,
            className,
            propertyName,
            projectType,
            dot, offsetFromRoot: workspace_1.offsetFromRoot(projectRoot) });
    }
    createCommand(commands) {
        return {
            builder: "@nrwl/workspace:run-commands",
            options: {
                commands: commands
            }
        };
    }
    getServeConfig(options = this.options) {
        const commands = [
            { command: `npx @google-cloud/functions-framework --target ${options.propertyName} --source ./dist/apps/${options.name}` }
        ];
        return this.createCommand(commands);
    }
    getDeployConfig(options = this.options) {
        const expr = this.options.trigger;
        let commands = [];
        switch (expr) {
            case '--trigger-http':
                commands.push({
                    command: `gcloud functions deploy ${options.propertyName} ${options.trigger} --runtime ${options.runtime} --region ${options.region} --env-vars-file ./dist/apps/${options.name}/.production.yaml --source ./dist/apps/${options.name} --max-instances ${options.maxInstances} --allow-unauthenticated`
                });
                break;
            case '--trigger-topic':
                const topic = options.triggerTopic.length ? options.triggerTopic : options.propertyName;
                commands.push({
                    command: `gcloud functions deploy ${options.propertyName} ${options.trigger} ${topic} --runtime ${options.runtime} --region ${options.region} --env-vars-file ./dist/apps/${options.name}/.production.yaml --source ./dist/apps/${options.name} --max-instances ${options.maxInstances} --allow-unauthenticated`
                });
                break;
        }
        return this.createCommand(commands);
    }
    getTestConfig(options = this.options) {
        return {
            builder: "@nrwl/jest:jest",
            options: {
                jestConfig: core_1.join(options.appProjectRoot, "jest.config.js"),
                tsConfig: core_1.join(options.appProjectRoot, "tsconfig.spec.json"),
                passWithNoTests: true
            }
        };
    }
    getBuildConfig(project = this.project, options = this.options) {
        return {
            builder: "@joelcode/gcp-function:build",
            options: {
                outputPath: core_1.join(core_1.normalize("dist"), options.appProjectRoot),
                main: core_1.join(project.sourceRoot, "index.ts"),
                yamlConfig: core_1.join(project.sourceRoot, "/environments/.production.yaml"),
                tsConfig: core_1.join(options.appProjectRoot, "tsconfig.app.json"),
                packageJson: core_1.join(options.appProjectRoot, "package.json"),
            },
            configurations: {
                production: {
                    optimization: true,
                    extractLicenses: true,
                    inspect: false
                }
            }
        };
    }
    getLintConfig(project = this.project) {
        return workspace_1.generateProjectLint(core_1.normalize(project.root), core_1.join(core_1.normalize(project.root), "tsconfig.app.json"), "eslint" /* EsLint */);
    }
    getProjectArchitect() {
        const expr = this.options.trigger;
        switch (expr) {
            case '--trigger-http':
                this.project.architect.serve = this.getServeConfig();
                break;
            case '--trigger-topic':
                break;
        }
        this.project.architect.lint = this.getLintConfig();
        this.project.architect.test = this.getTestConfig();
        this.project.architect.build = this.getBuildConfig();
        this.project.architect.deploy = this.getDeployConfig();
        return this.project;
    }
    updateWorkspaceJson(options) {
        return workspace_1.updateWorkspaceInTree(workspaceJson => {
            workspaceJson.projects[options.name] = this.getProjectArchitect();
            workspaceJson.defaultProject = workspaceJson.defaultProject || options.name;
            return workspaceJson;
        });
    }
    log(message) {
        this.context.logger.info(util_1.inspect(message, false, null));
    }
}
exports.default = ProjectTools;
//# sourceMappingURL=projectTools.js.map